module ServiceLayerRest
  class AuthorizationsController < ApplicationController
    include Base
    include Dll

    def login
      invalid_key = check_params([:username, :password], [:mandantid])
      invalid_key = :authtype if params[:authtype].nil? && invalid_key.blank?
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.put_login(
          params[:username],
          params[:password],
          params[:authtype],
          params[:mandantid].to_i
        )
        ServiceLayerDll.free_string(string_pointer)

        render render_json = parse_response_string_for_render(response_string, {"version" => version})
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def get_employee_rights
      invalid_key = check_params([:fct_code], [])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.get_employee_rights(
          current_user.as_json,
          params[:fct_code]
        )
        ServiceLayerDll.free_string(string_pointer)

        render render_json = parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def get_employee_rights_userconfig
      invalid_key = check_params([:fct_code], [])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.get_employee_rights_userconfig(
          current_user.as_json,
          params[:fct_code]
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def get_permission
      invalid_key = check_params([], [:given_fct_code, :min_right, :mandant_id, :planarea_id, :subjectarea_id])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.get_permission(
          current_user.as_json,
          params[:given_fct_code].to_i,
          params[:min_right].to_i,
          params[:mandant_id].to_i,
          params[:planarea_id].to_i,
          params[:subjectarea_id].to_i
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def get_permission_right
      invalid_key = check_params([], [:given_fct_code, :mandant_id, :planarea_id, :subjectarea_id])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.get_permission_right(
          current_user.as_json,
          params[:given_fct_code].to_i,
          params[:mandant_id].to_i,
          params[:planarea_id].to_i,
          params[:subjectarea_id].to_i
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end

    def get_permitted_planareas
      invalid_key = check_params([], [:given_fct_code])
      return render bad_params_for_render(invalid_key) unless invalid_key.blank?

      begin
        response_string, string_pointer = ServiceLayerDll.get_permitted_planareas(
          current_user.as_json,
          params[:given_fct_code].to_i
        )
        ServiceLayerDll.free_string(string_pointer)

        render parse_response_string_for_render(response_string)
      rescue Exception => e
        render parse_exception_for_render(e)
      end
    end
  end
end
